package gs.uuid.v0

class UUIDTests extends munit.FunSuite:
  private val v4                                 = UUID.Generator.version4
  private val v7                                 = UUID.Generator.version7
  given CanEqual[java.util.UUID, java.util.UUID] = CanEqual.derived

  test(
    "should instantiate a type 4 UUID, serialize it, and parse the result"
  ) {
    val base   = v4.next()
    val str    = base.str()
    val parsed = UUID.parse(str)
    assertEquals(parsed, Some(base))
  }

  test(
    "should instantiate a type 4 UUID, serialize it, and parse the result (helper function)"
  ) {
    val base   = UUID.v4()
    val str    = base.str()
    val parsed = UUID.parse(str)
    assertEquals(parsed, Some(base))
  }

  test(
    "should instantiate a type 7 UUID, serialize it, and parse the result"
  ) {
    val base   = v7.next()
    val str    = base.str()
    val parsed = UUID.parse(str)
    assertEquals(parsed, Some(base))
  }

  test(
    "should instantiate a type 7 UUID, serialize it, and parse the result (helper function)"
  ) {
    val base   = UUID.v7()
    val str    = base.str()
    val parsed = UUID.parse(str)
    assertEquals(parsed, Some(base))
  }

  test("should instantiate from any java.util.UUID") {
    val raw    = java.util.UUID.randomUUID()
    val base   = UUID(raw)
    val str    = base.str()
    val parsed = UUID.fromString(str)
    assertEquals(parsed, Some(base))
    assertEquals(parsed.map(_.toUUID()), Some(raw))
  }

  test("should successfully parse a UUID with dashes") {
    val raw  = java.util.UUID.randomUUID()
    val base = UUID(raw)
    assertEquals(UUID.parse(base.withDashes()), Some(base))
  }

  test(
    "should successfully parse a UUID with dashes, generated with this library"
  ) {
    val base = java.util.UUID.randomUUID()
    assertEquals(UUID.parse(base.toString()), Some(UUID(base)))
  }

  test("should fail to parse a non-hex string") {
    val input = "ghijklmnoped45189efa6f40ed9518ed"
    assertEquals(UUID.parse(input), None)
  }

  test("should round trip to/from a byte array") {
    given UUID.Generator = v7
    val base             = UUID.generate()
    val bytes            = base.toBytes()
    val parsed           = UUID.fromBytes(bytes)
    assertEquals(parsed, Some(base))
  }

  test("should fail to parse an invalid byte array") {
    val bytes = Array[Byte](1)
    assertEquals(UUID.fromBytes(bytes), None)
  }

  test("should generate using an available type class instance") {
    given UUID.Generator = v7
    val base             = doGen
    val str              = base.str()
    val parsed           = UUID.parse(str)
    assertEquals(parsed, Some(base))
  }

  test("should return lsb, msb, and if the UUID is zero") {
    val uuid = UUID.parse("00000000-0000-0000-0000-000000000000")
    assertEquals(uuid.map(_.lsb()), Some(0L))
    assertEquals(uuid.map(_.msb()), Some(0L))
    assertEquals(uuid.map(_.isZero()), Some(true))
  }

  private def doGen(
    using
    UUID.Generator
  ): UUID =
    UUID.generate()
